const fsPromise = require("fs/promises");

const IDGEN_URI = require('../config.js').IDGEN_URI
const Ks3Util = require('../utils/ksUtil').Ks3Util
const KS3_CONST = require('../constant').KS3_CONST
const req = require("../utils/request").httpReq;
const FormData = require('form-data');


const _get_obj_id = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/buildObectid`;
  var opts = {
    url: url,
    qs: {
      appid: 'merchant-b',
    },
    method: 'GET',
  }
  return await req(ctx, opts);
}

const _get_bucket = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/getBucket`;
  var opts = {
    url: url,
    qs: {
      appid: 'merchant-b',
    },
    method: 'GET',
  }
  // console.log(await req(ctx, opts))
  return await req(ctx, opts)
}

const _get_token = async (ctx, next) => {
  var url = `${IDGEN_URI}/Ksy/ksyun/getToken`;
  var opts = {
    url: url,
    data: {
      appid: 'merchant-b',
    },
    method: 'POST',
  }
  return await req(ctx, opts)
}


exports.get_obj_id = async (ctx, next) => {
  ctx.body = _get_obj_id(ctx, next)
}


exports.get_bucket = async (ctx, next) => {
  ctx.body = await _get_bucket(ctx, next)
}

exports.get_token = async (ctx, next) => {
  ctx.body = await _get_token(ctx, next)
}

exports.get_ks3_config = async (ctx, next) => {
  const { result: { bucket }} = await _get_bucket(ctx, next)
  const { result: { objectId }} = await _get_obj_id(ctx, next)

  const getExpires = (seconds) => {
    return Math.round(new Date().getTime()/1000) + seconds;
  };

  const policy = {
    "expiration": new Date(getExpires(3600)*1000).toISOString(), //一小时后
    "conditions": [
      ["eq","$bucket", bucket],
      ["starts-with","$acl", "public-read"],
      // ["starts-with", "$key", ""],
      // ["starts-with", "$name", ""],   //表单中传了name字段，也需要加到policy中
      // ["starts-with", "$x-kss-meta-custom-param1",""],
      // ["starts-with", "$x-kss-newfilename-in-body",""],//必须只包含小写字符
      // ["starts-with", "$Cache-Control",""],
      // ["starts-with", "$Expires", ""],
      // ["starts-with", "$Content-Disposition", ""],
      // ["starts-with", "$Content-Type",""],
      // ["starts-with", "$Content-Encoding",""]
    ]
  }
  const stringToSign = Ks3Util.Base64.encode(JSON.stringify(policy))

  console.log('stringToSign:::', stringToSign)
  const signature = Ks3Util.b64_hmac_sha1(KS3_CONST.AccessKeySecret, stringToSign)

  ctx.body = {
    bucket,
    objectId,
    policy: stringToSign,
    signature,
  }
}

exports.upload_ks3_image = async (ctx, next) => {

  // console.log('upload_ks3_image::::::', ctx.request.files)
  const file = ctx.request.files && ctx.request.files.file;
  // console.log(34444, file)
  // const bucketOpt = {
  //   url: `${IDGEN_URI}/Ksy/ksyun/getBucket`,
  //   params: {
  //     appid: 'merchant-b',
  //   },
  //   method: 'GET',
  //   timeout: 8000,
  // }

  // const objIdOpt = {
  //   url: `${IDGEN_URI}/Ksy/ksyun/buildObectid`,
  //   params: {
  //     appid: 'merchant-b',
  //   },
  //   method: 'GET',
  //   timeout: 8000,
  // }

  // const { result: { bucket }} = await _get_bucket(ctx, next)
  // const { result: { objectId }} = await _get_obj_id(ctx, next)
  const { result: { bucket }} = await _get_bucket(ctx, next)
  const { result: { objectId }} = await _get_obj_id(ctx, next)
  console.log(3334444, bucket, objectId, file,)

  const getExpires = (seconds) => {
    return Math.round(new Date().getTime()/1000) + seconds;
  };

  const policy = {
    "expiration": new Date(getExpires(3600)*1000).toISOString(), //一小时后
    "conditions": [
      ["eq","$bucket", bucket],
      ["starts-with","$acl", "public-read"],
    ]
  }
  const stringToSign = Ks3Util.Base64.encode(JSON.stringify(policy))

  console.log('stringToSign:::', stringToSign)
  const signature = Ks3Util.b64_hmac_sha1(KS3_CONST.AccessKeySecret, stringToSign)
  // const reader = fs.createReadStream(file.path);

  const reader = await fsPromise.readFile(file.path);

  const formData = new FormData()
  formData.append('acl', 'public-read')
  formData.append('key', objectId)
  formData.append('signature', signature)
  formData.append('KSSAccessKeyId', KS3_CONST.AccessKeyID)
  formData.append('policy', stringToSign)
  formData.append('bucket_name', stringToSign)
  formData.append('file', reader)
  // axios.post(OSS_URL, formData).then(res => {
  //   const { status } = res
  //   if (status === 200) {
  //     const data = {
  //       url: `${OSS_URL}/${key}`,
  //       type: fileType(file.name)
  //     }
  //     resolve(data)
  //   } else {
  //     reject(res)
  //   })

  var opts = {
    url: `http://${KS3_CONST.DOMAIN}/${bucket}`,
    method: 'POST',
    headers: {
      // 'content-type': 'multipart/form-data'
      ...formData.getHeaders(),
      'content-length': formData.getLengthSync(),
    },
    data: formData
  }
  console.log(3333333333, formData.getLengthSync());
  const res = await req(ctx, opts);
  console.log('sever ks3 image::::::', res)
  ctx.body = res;
}